@@ -11,7 +11,7 @@ my $attr = $factory->createAttribute ({
 	attribute_name => 'name',
 	short_description => 'the name of the athlete',
 });
-$bean->addAttribute ($attr->getAttributeName (), $attr);
+$bean->addAttribute ($attr);
 
 my $bean2 = PerlBean->new ({
 	package => 'Cyclist',
@@ -22,10 +22,10 @@ my $attr2 = $factory->createAttribute ({
 	attribute_name => 'cycle',
 	short_description => 'the cyclist\'s cycle',
 });
-$bean2->addAttribute ($attr2->getAttributeName (), $attr2);
+$bean2->addAttribute ($attr2);
 
 my $collection = PerlBean::Collection->new ();
-$collection->addBean ($bean->getPackage (), $bean);
-$collection->addBean ($bean2->getPackage (), $bean2);
+$collection->addPerlBean ($bean);
+$collection->addPerlBean ($bean2);
 $collection->write ('tmp');
 1;
