@@ -8,7 +8,7 @@ use AutoLoader qw (AUTOLOAD);
 
 use base qw (PerlBean::Attribute::Multi);
 
-our ( $VERSION ) = '$Revision: 0.2.0.2 $ ' =~ /\$Revision:\s+([^\s]+)/;
+our ( $VERSION ) = '$Revision: 0.3 $ ' =~ /\$Revision:\s+([^\s]+)/;
 
 our $SUB = 'sub';
 
@@ -115,17 +115,67 @@ Passed to L<setAllowValue ()>. Must be an C<ARRAY> reference.
 
 =head1 METHODS
 
-Inherited/implemented from L<PerlBean::Attribute>.
-
 =over
 
-=item writeMethods (FILEHANDLE)
+=item getOverloadedAttribute ()
 
-Writes code for B<set>, B<push>, B<pop>, B<shift>, B<unshift>, B<exists> and B<get> methods.
+This method is inherited from package C<'PerlBean::Attribute'>. Searches superclass packages for an identically named C<PerlBean::Attribute>. If found it is returned otherwise C<undef> is returned
+
+=item getPackage ()
+
+This method is inherited from package C<'PerlBean::Attribute'>. Returns the package name. The package name is obtained from the C<PerlBean> to which the C<PerlBean::Attribute> belongs. Or, if the C<PerlBean::Attribute> does not belong to a C<PerlBean>, C<main> is returned.
+
+=item getPackageUS ()
+
+This method is inherited from package C<'PerlBean::Attribute'>. Calls C<getPackage ()> and replaces C<:+> with C <_>.
+
+=item type ()
+
+This method is inherited from package C<'PerlBean::Attribute'>. Determins and returns the type of the attribute. The type is either C<BOOLEAN>, C<SINGLE> or C<MULTI>.
+
+=item writeAllowIsa (FILEHANDLE)
+
+This method is inherited from package C<'PerlBean::Attribute::Single'>. Writes C<%ALLOW_ISA> line for the attribute. C<FILEHANDLE> is an C<IO::Handle> object.
+
+=item writeAllowRef (FILEHANDLE)
+
+This method is inherited from package C<'PerlBean::Attribute::Single'>. Writes C<%ALLOW_REF> line for the attribute. C<FILEHANDLE> is an C<IO::Handle> object.
+
+=item writeAllowRx (FILEHANDLE)
+
+This method is inherited from package C<'PerlBean::Attribute::Single'>. Writes C<%ALLOW_RX> line for the attribute. C<FILEHANDLE> is an C<IO::Handle> object.
+
+=item writeAllowValue (FILEHANDLE)
+
+This method is inherited from package C<'PerlBean::Attribute::Single'>. Writes C<%ALLOW_VALUE> line for the attribute. C<FILEHANDLE> is an C<IO::Handle> object.
+
+=item writeDefaultValue (FILEHANDLE)
+
+This method is inherited from package C<'PerlBean::Attribute::Multi'>. Writes C<%DEFAULT_VALUE> line for the attribute. C<FILEHANDLE> is an C<IO::Handle> object.
+
+=item writeDocClauses (FILEHANDLE)
+
+This method is inherited from package C<'PerlBean::Attribute'>. Writes documentation for the clauses to which the contents the contents of the attribute must adhere. C<FILEHANDLE> is an C<IO::Handle> object.
+
+=item writeDocInheritMethods (FILEHANDLE)
+
+This method is overloaded from package C<'PerlBean::Attribute::Multi'>. Writes documentation for the access methods for the attribute in the case the attibute methods are inherited. C<FILEHANDLE> is an C<IO::Handle> object. As package C<PerlBean::Attribute::Multi> is intended to group its subclasses this method actually throws an exception. Access methods are B<set...>, B<setIdx...>, B<setNum...>, B<push...>, B<pop...>, B<shift...>, B<unshift...>, B<exists...> and B<get...>.
+
+=item writeDocInit (FILEHANDLE)
+
+This method is inherited from package C<'PerlBean::Attribute::Multi'>. Writes documentation for C<_initialize ()> for the attribute. C<FILEHANDLE> is an C<IO::Handle> object.
 
 =item writeDocMethods (FILEHANDLE)
 
-Writes documenation for B<set>, B<push>, B<pop>, B<shift>, B<unshift>, B<exists> and B<get> methods.
+This method is overloaded from package C<'PerlBean::Attribute::Multi'>. Writes documentation for the access methods for the attribute. C<FILEHANDLE> is an C<IO::Handle> object. As package C<PerlBean::Attribute::Multi> is intended to group its subclasses this method actually throws an exception. Access methods are B<set...>, B<setIdx...>, B<setNum...>, B<push...>, B<pop...>, B<shift...>, B<unshift...>, B<exists...> and B<get...>.
+
+=item writeMethods (FILEHANDLE)
+
+This method is overloaded from package C<'PerlBean::Attribute::Multi'>. Writes the access methods for the attribute. C<FILEHANDLE> is an C<IO::Handle> object. As package C<PerlBean::Attribute::Multi> is intended to group its subclasses this method actually throws an exception. Access methods are B<set...>, B<setIdx...>, B<setNum...>, B<push...>, B<pop...>, B<shift...>, B<unshift...>, B<exists...> and B<get...>.
+
+=item writeOptInit (FILEHANDLE)
+
+This method is inherited from package C<'PerlBean::Attribute::Multi'>. Writes C<_initialize ()> option parsing code for the attribute. C<FILEHANDLE> is an C<IO::Handle> object.
 
 =back
 
@@ -198,9 +248,11 @@ L<PerlBean::Attribute::Factory>,
 L<PerlBean::Attribute::Multi>,
 L<PerlBean::Attribute::Multi::Unique>,
 L<PerlBean::Attribute::Multi::Unique::Associative>,
+L<PerlBean::Attribute::Multi::Unique::Associative::MethodKey>,
 L<PerlBean::Attribute::Multi::Unique::Ordered>,
 L<PerlBean::Attribute::Single>,
-L<PerlBean::Collection>
+L<PerlBean::Collection>,
+L<PerlBean::Method>
 
 =head1 BUGS
 
@@ -216,7 +268,7 @@ Vincenzo Zocca
 
 =head1 COPYRIGHT
 
-Copyright 2002 by Vincenzo Zocca
+Copyright 2002, 2003 by Vincenzo Zocca
 
 =head1 LICENSE
 
@@ -240,19 +292,22 @@ Boston, MA 02111-1307 USA
 
 =cut
 
-sub writeMethods {
+sub writeDocInheritMethods {
 	my $self = shift;
 	my $fh = shift;
 
-	$self->writeSetMethod ($fh);
-	$self->writeSetIdxMethod ($fh);
-	$self->writeSetNumMethod ($fh);
-	$self->writePushMethod ($fh);
-	$self->writePopMethod ($fh);
-	$self->writeShiftMethod ($fh);
-	$self->writeUnshiftMethod ($fh);
-	$self->writeExistsMethod ($fh);
-	$self->writeGetMethod ($fh);
+	my $mb = $self->getMethodBase ();
+
+	my @meth = ();
+	foreach my $op (qw (set setIdx setNum push pop shift unshift exists get)) {
+		push (@meth, "$op$mb ()");
+	}
+	my $meth = join (', ', @meth);
+
+	$fh->print (<<EOF);
+$meth
+
+EOF
 }
 
 sub writeDocMethods {
@@ -270,22 +325,19 @@ sub writeDocMethods {
 	$self->writeGetDoc ($fh);
 }
 
-sub writeDocInheritMethods {
+sub writeMethods {
 	my $self = shift;
 	my $fh = shift;
 
-	my $mb = $self->getMethodBase ();
-
-	my @meth = ();
-	foreach my $op (qw (set setIdx setNum push pop shift unshift exists get)) {
-		push (@meth, "$op$mb ()");
-	}
-	my $meth = join (', ', @meth);
-
-	$fh->print (<<EOF);
-$meth
-
-EOF
+	$self->writeSetMethod ($fh);
+	$self->writeSetIdxMethod ($fh);
+	$self->writeSetNumMethod ($fh);
+	$self->writePushMethod ($fh);
+	$self->writePopMethod ($fh);
+	$self->writeShiftMethod ($fh);
+	$self->writeUnshiftMethod ($fh);
+	$self->writeExistsMethod ($fh);
+	$self->writeGetMethod ($fh);
 }
 
 sub writeSetMethod {
